package cz.muni.pa165.authentication;

import cz.muni.pa165.common_library.exception.RestExceptionHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.autoconfigure.domain.EntityScan;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Import;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.web.SecurityFilterChain;

/**
 * Main Authentication App.
 */
@SpringBootApplication
@EntityScan(basePackages = {"cz.muni.pa165.authentication.data.model"})
@Import({RestExceptionHandler.class})
public class App {

  private static final Logger log = LoggerFactory.getLogger(App.class);

  public static void main(String[] args) {
    SpringApplication.run(App.class, args);
  }

  /**
   * Security filter chain for roles.
   *
   * @param httpSecurity httpsecurity
   * @return filter chain
   * @throws Exception when error occurs
   */
  @Bean
  public SecurityFilterChain securityFilterChain(HttpSecurity httpSecurity) throws Exception {
    httpSecurity
        .csrf().disable()
        .authorizeHttpRequests(x -> x
            .requestMatchers("/error", "/robots.txt", "/style.css", "/favicon.ico",
                "/webjars/**", "/authenticate/apikey", "", "/authenticate/apikey/getManagerEmails")
            .permitAll()
            .requestMatchers("/authenticate/apikey/deleteAll", "/apikey/delete")
            .hasAnyAuthority("NOONE")
            .anyRequest().authenticated()
        )
        .logout(x -> x.logoutSuccessUrl("/"));

    return httpSecurity.build();
  }

}
